--
-- Save my Position Script
-- author: Blacky_BPG
-- 
-- can save the last position on a map and set the player to this position after loading an savegame or join a mp session
-- 
-- Version 1.0.0 | 20.04.2014 | initial release
-- Version 1.1.0 | 28.04.2014 | fixed map title dot replacer and added position marker
-- 

samypo = {};
samypo_dir = nil;
samypo_file = nil;
samypo_mapTitle = nil;

function samypo:loadMap(name)
	samypo_dir = getUserProfileAppPath() .. "mods_positionData";
	createFolder(samypo_dir)
	self.samypoLoaded = false;
	g_currentMission.samypoAllowed = false;
	self.samypoWorking = true;
	self.samypoHotSpot = nil;
end;

function samypo:deleteMap()
	if self.samypoHotSpot ~= nil then
		g_currentMission.missionPDA:deleteMapHotspot(self.samypoHotSpot);
	end;
end;

function samypo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function samypo:keyEvent(unicode, sym, modifier, isDown)
end;

function samypo:draw()
end;

function samypo:update(dt)
	if self.samypoWorking then
		local iconSize = g_currentMission.missionPDA.pdaMapWidth / 15;

		-- correct map title errors
		if samypo_mapTitle == nil or string.len(samypo_mapTitle) < 3 then
			local tmp1, tmp2 = nil, nil;
			local mapName = "";
			for i=1, string.len(g_currentMission.missionInfo.map.title) do
				local tmp = string.sub(g_currentMission.missionInfo.map.title,i,i);
				if tmp == " " then tmp = "_" end;
				if tmp == "." then tmp = "_" end;
				mapName = mapName .. tmp;
			end;
			if mapName ~= nil and string.len(mapName) > 3 then
				samypo_mapTitle = mapName;
			else
				if mapName == nil then
					print("Error: can not read Map Title, SaMyPo can not work !");
				elseif string.len(mapName) < 4 then
					print("Error: Map Title are to short, minimum length is 4 strings, SaMyPo can not work !");
				end;
				self.samypoWorking = false;
			end;
		end;

		-- loading module
		if not self.samypoLoaded then
			if g_currentMission.player ~= nil then
				samypo_file = samypo_dir .. "/" .. samypo_mapTitle .. ".xml";
				self.samypoLoaded = true;
				local x,y,z = getWorldTranslation(g_currentMission.player.rootNode);
				local rotX, rotY, rotZ = getWorldRotation(g_currentMission.player.rootNode);
				if fileExists(samypo_file) then
					local xmlFile = loadXMLFile(samypo_mapTitle.."_samypo", samypo_file);
					if xmlFile ~= nil then
						setPositionForPlayer = true;
						x = Utils.getNoNil(getXMLFloat(xmlFile, samypo_mapTitle.."_samypo".."#x"), x);
						y = Utils.getNoNil(getXMLFloat(xmlFile, samypo_mapTitle.."_samypo".."#y"), y);
						z = Utils.getNoNil(getXMLFloat(xmlFile, samypo_mapTitle.."_samypo".."#z"), z);
						delete(xmlFile);
						g_currentMission.player:moveToAbsolute(x, y, z);
						g_client:getServerConnection():sendEvent(PlayerTeleportEvent:new(x, y, z))
						print("Success: Player position set to (x,y,z) "..tostring(x).." , "..tostring(y).." , "..tostring(z));
						if self.samypoHotSpot ~= nil then
							g_currentMission.missionPDA:deleteMapHotspot(self.samypoHotSpot);
						end;
						self.samypoHotSpot = g_currentMission.missionPDA:createMapHotspot("SaMyPo", Utils.getFilename("$dataS2/menu/cross_hair.dds", self.baseDirectory),x,z, iconSize, iconSize * (4 / 3), false, false, false, 0, true);
					end;
				end;
			end;
		end;

		-- save module
		-- can be blocked if a samypo maptrigger added
		if g_currentMission ~= nil and g_currentMission.player ~= nil and g_currentMission.player.isEntered and g_currentMission.player.isControlled then
			if g_currentMission.samypoAllowed or (InputBinding.isPressed(InputBinding.samypoFirst) and not g_currentMission.samypoTrigger) then 
				g_currentMission:addHelpButtonText(g_i18n:getText("samypoOption"), InputBinding.samypoSecond);
				if InputBinding.hasEvent(InputBinding.samypoSecond) then
					if samypo_file ~= nil then
						local xmlFile = createXMLFile(samypo_mapTitle.."_samypo", samypo_file, samypo_mapTitle.."_samypo");
						local x,y,z = getWorldTranslation(g_currentMission.player.rootNode);
						local rotX, rotY, rotZ = getWorldRotation(g_currentMission.player.rootNode);
						if xmlFile ~= nil then
							setXMLFloat(xmlFile,samypo_mapTitle.."_samypo".."#x",x);
							setXMLFloat(xmlFile,samypo_mapTitle.."_samypo".."#y",y);
							setXMLFloat(xmlFile,samypo_mapTitle.."_samypo".."#z",z);
							saveXMLFile(xmlFile);
							print("Success: Player position saved to (x,y,z) "..tostring(x).." , "..tostring(y).." , "..tostring(z).." in SaMyPo-File");
							if self.samypoHotSpot ~= nil then
								g_currentMission.missionPDA:deleteMapHotspot(self.samypoHotSpot);
							end;
							self.samypoHotSpot = g_currentMission.missionPDA:createMapHotspot("SaMyPo", Utils.getFilename("$dataS2/menu/cross_hair.dds", self.baseDirectory),x,z, iconSize, iconSize * (4 / 3), false, false, false, 0, true);
						else
							print("Error: can not save the file "..tostring(samypo_file).." ("..tostring(xmlFile)..") ".." as a file error has occurred");
						end;
					end;
				end;
			end;
		end;
	else
		if self.samypoHotSpot ~= nil then
			g_currentMission.missionPDA:deleteMapHotspot(self.samypoHotSpot);
		end;
	end;
end;

addModEventListener(samypo);

